@@ -655,6 +655,34 @@ class AtomTest(CSTNodeTest):
                 "parser": _parse_expression_force_38,
                 "expected_position": None,
             },
+            {
+                "node": cst.FormattedString(
+                    parts=(
+                        cst.FormattedStringExpression(
+                            cst.Yield(
+                                value=cst.Integer("1"),
+                                whitespace_after_yield=cst.SimpleWhitespace(" "),
+                            ),
+                        ),
+                    ),
+                ),
+                "code": 'f"{yield 1}"',
+                "parser": _parse_expression_force_38,
+                "expected_position": None,
+            },
+            {
+                "node": cst.FormattedString(
+                    parts=(
+                        cst.FormattedStringText("\\N{X Y}"),
+                        cst.FormattedStringExpression(
+                            cst.Name(value="Z"),
+                        ),
+                    ),
+                ),
+                "code": 'f"\\N{X Y}{Z}"',
+                "parser": parse_expression,
+                "expected_position": None,
+            },
             # Validate parens
             {
                 "node": cst.FormattedString(
