@@ -15,13 +15,13 @@ import sys
 import time
 import traceback
 from dataclasses import dataclass, replace
-from multiprocessing import Process, Queue, cpu_count
+from multiprocessing import Pool, cpu_count
 from pathlib import Path
-from queue import Empty
-from typing import AnyStr, Dict, List, Optional, Sequence, Set, Union, cast
+from typing import Any, AnyStr, Dict, List, Optional, Sequence, Union, cast
 
 from libcst import PartialParserConfig, parse_module
 from libcst.codemod._codemod import Codemod
+from libcst.codemod._dummy_pool import DummyPool
 from libcst.codemod._runner import (
     SkipFile,
     SkipReason,
@@ -216,7 +216,7 @@ def _calculate_module(repo_root: Optional[str], filename: str) -> Optional[str]:
 
 
 @dataclass(frozen=True)
-class ParallelExecResult:
+class ExecutionResult:
     # File we have results for
     filename: str
     # Whether we actually changed the code for the file or not
@@ -225,50 +225,51 @@ class ParallelExecResult:
     transform_result: TransformResult
 
 
-def _parallel_exec_process_stub(  # noqa: C901
-    result_queue: "Queue[ParallelExecResult]",
+@dataclass(frozen=True)
+class ExecutionConfig:
+    blacklist_patterns: Sequence[str] = ()
+    format_code: bool = False
+    formatter_args: Sequence[str] = ()
+    generated_code_marker: str = _DEFAULT_GENERATED_CODE_MARKER
+    include_generated: bool = False
+    python_version: Optional[str] = None
+    repo_root: Optional[str] = None
+    unified_diff: Optional[int] = None
+
+
+def _execute_transform(  # noqa: C901
     transformer: Codemod,
     filename: str,
-    repo_root: Optional[str],
-    unified_diff: Optional[int],
-    include_generated: bool,
-    generated_code_marker: str,
-    format_code: bool,
-    formatter_args: Sequence[str],
-    blacklist_patterns: Sequence[str],
-    python_version: Optional[str],
-) -> None:
-    for pattern in blacklist_patterns:
+    config: ExecutionConfig,
+) -> ExecutionResult:
+    for pattern in config.blacklist_patterns:
         if re.fullmatch(pattern, filename):
-            result_queue.put(
-                ParallelExecResult(
-                    filename=filename,
-                    changed=False,
-                    transform_result=TransformSkip(
-                        skip_reason=SkipReason.BLACKLISTED,
-                        skip_description=f"Blacklisted by pattern {pattern}.",
-                    ),
-                )
+            return ExecutionResult(
+                filename=filename,
+                changed=False,
+                transform_result=TransformSkip(
+                    skip_reason=SkipReason.BLACKLISTED,
+                    skip_description=f"Blacklisted by pattern {pattern}.",
+                ),
             )
-            return
 
     try:
         with open(filename, "rb") as fp:
             oldcode = fp.read()
 
         # Skip generated files
-        if not include_generated and generated_code_marker.encode("utf-8") in oldcode:
-            result_queue.put(
-                ParallelExecResult(
-                    filename=filename,
-                    changed=False,
-                    transform_result=TransformSkip(
-                        skip_reason=SkipReason.GENERATED,
-                        skip_description="Generated file.",
-                    ),
-                )
+        if (
+            not config.include_generated
+            and config.generated_code_marker.encode("utf-8") in oldcode
+        ):
+            return ExecutionResult(
+                filename=filename,
+                changed=False,
+                transform_result=TransformSkip(
+                    skip_reason=SkipReason.GENERATED,
+                    skip_description="Generated file.",
+                ),
             )
-            return
 
         # Somewhat gross hack to provide the filename in the transform's context.
         # We do this after the fork so that a context that was initialized with
@@ -277,7 +278,7 @@ def _parallel_exec_process_stub(  # noqa: C901
         transformer.context = replace(
             transformer.context,
             filename=filename,
-            full_module_name=_calculate_module(repo_root, filename),
+            full_module_name=_calculate_module(config.repo_root, filename),
         )
 
         # Run the transform, bail if we failed or if we aren't formatting code
@@ -285,8 +286,8 @@ def _parallel_exec_process_stub(  # noqa: C901
             input_tree = parse_module(
                 oldcode,
                 config=(
-                    PartialParserConfig(python_version=python_version)
-                    if python_version is not None
+                    PartialParserConfig(python_version=str(config.python_version))
+                    if config.python_version is not None
                     else PartialParserConfig()
                 ),
             )
@@ -294,28 +295,43 @@ def _parallel_exec_process_stub(  # noqa: C901
             newcode = output_tree.bytes
             encoding = output_tree.encoding
         except KeyboardInterrupt:
-            result_queue.put(
-                ParallelExecResult(
-                    filename=filename, changed=False, transform_result=TransformExit()
-                )
+            return ExecutionResult(
+                filename=filename, changed=False, transform_result=TransformExit()
             )
-            return
         except SkipFile as ex:
-            result_queue.put(
-                ParallelExecResult(
+            return ExecutionResult(
+                filename=filename,
+                changed=False,
+                transform_result=TransformSkip(
+                    skip_reason=SkipReason.OTHER,
+                    skip_description=str(ex),
+                    warning_messages=transformer.context.warnings,
+                ),
+            )
+        except Exception as ex:
+            return ExecutionResult(
+                filename=filename,
+                changed=False,
+                transform_result=TransformFailure(
+                    error=ex,
+                    traceback_str=traceback.format_exc(),
+                    warning_messages=transformer.context.warnings,
+                ),
+            )
+
+        # Call formatter if needed, but only if we actually changed something in this
+        # file
+        if config.format_code and newcode != oldcode:
+            try:
+                newcode = invoke_formatter(config.formatter_args, newcode)
+            except KeyboardInterrupt:
+                return ExecutionResult(
                     filename=filename,
                     changed=False,
-                    transform_result=TransformSkip(
-                        skip_reason=SkipReason.OTHER,
-                        skip_description=str(ex),
-                        warning_messages=transformer.context.warnings,
-                    ),
+                    transform_result=TransformExit(),
                 )
-            )
-            return
-        except Exception as ex:
-            result_queue.put(
-                ParallelExecResult(
+            except Exception as ex:
+                return ExecutionResult(
                     filename=filename,
                     changed=False,
                     transform_result=TransformFailure(
@@ -324,44 +340,14 @@ def _parallel_exec_process_stub(  # noqa: C901
                         warning_messages=transformer.context.warnings,
                     ),
                 )
-            )
-            return
-
-        # Call formatter if needed, but only if we actually changed something in this
-        # file
-        if format_code and newcode != oldcode:
-            try:
-                newcode = invoke_formatter(formatter_args, newcode)
-            except KeyboardInterrupt:
-                result_queue.put(
-                    ParallelExecResult(
-                        filename=filename,
-                        changed=False,
-                        transform_result=TransformExit(),
-                    )
-                )
-                return
-            except Exception as ex:
-                result_queue.put(
-                    ParallelExecResult(
-                        filename=filename,
-                        changed=False,
-                        transform_result=TransformFailure(
-                            error=ex,
-                            traceback_str=traceback.format_exc(),
-                            warning_messages=transformer.context.warnings,
-                        ),
-                    )
-                )
-                return
 
         # Format as unified diff if needed, otherwise save it back
         changed = oldcode != newcode
-        if unified_diff:
+        if config.unified_diff:
             newcode = diff_code(
                 oldcode.decode(encoding),
                 newcode.decode(encoding),
-                unified_diff,
+                config.unified_diff,
                 filename=filename,
             )
         else:
@@ -373,32 +359,26 @@ def _parallel_exec_process_stub(  # noqa: C901
             newcode = ""
 
         # Inform success
-        result_queue.put(
-            ParallelExecResult(
-                filename=filename,
-                changed=changed,
-                transform_result=TransformSuccess(
-                    warning_messages=transformer.context.warnings, code=newcode
-                ),
-            )
+        return ExecutionResult(
+            filename=filename,
+            changed=changed,
+            transform_result=TransformSuccess(
+                warning_messages=transformer.context.warnings, code=newcode
+            ),
         )
     except KeyboardInterrupt:
-        result_queue.put(
-            ParallelExecResult(
-                filename=filename, changed=False, transform_result=TransformExit()
-            )
+        return ExecutionResult(
+            filename=filename, changed=False, transform_result=TransformExit()
         )
     except Exception as ex:
-        result_queue.put(
-            ParallelExecResult(
-                filename=filename,
-                changed=False,
-                transform_result=TransformFailure(
-                    error=ex,
-                    traceback_str=traceback.format_exc(),
-                    warning_messages=transformer.context.warnings,
-                ),
-            )
+        return ExecutionResult(
+            filename=filename,
+            changed=False,
+            transform_result=TransformFailure(
+                error=ex,
+                traceback_str=traceback.format_exc(),
+                warning_messages=transformer.context.warnings,
+            ),
         )
 
 
@@ -465,7 +445,7 @@ class Progress:
 
 
 def _print_parallel_result(
-    exec_result: ParallelExecResult,
+    exec_result: ExecutionResult,
     progress: Progress,
     *,
     unified_diff: bool,
@@ -534,6 +514,13 @@ class ParallelTransformResult:
     skips: int
 
 
+# Unfortunate wrapper required since there is no `istarmap_unordered`...
+def _execute_transform_wrap(
+    job: Dict[str, Any],
+) -> ExecutionResult:
+    return _execute_transform(**job)
+
+
 def parallel_exec_transform_with_prettyprint(  # noqa: C901
     transform: Codemod,
     files: Sequence[str],
@@ -590,7 +577,7 @@ def parallel_exec_transform_with_prettyprint(  # noqa: C901
     progress = Progress(enabled=not hide_progress, total=total)
 
     # Grab number of cores if we need to
-    jobs: int = jobs or cpu_count()
+    jobs: int = jobs if jobs is not None else cpu_count()
 
     if jobs < 1:
         raise Exception("Must have at least one job to process!")
@@ -598,7 +585,7 @@ def parallel_exec_transform_with_prettyprint(  # noqa: C901
     if total == 0:
         return ParallelTransformResult(successes=0, failures=0, skips=0, warnings=0)
 
-    if repo_root:
+    if repo_root is not None:
         # Make sure if there is a root that we have the absolute path to it.
         repo_root = os.path.abspath(repo_root)
         # Spin up a full repo metadata manager so that we can provide metadata
@@ -616,171 +603,75 @@ def parallel_exec_transform_with_prettyprint(  # noqa: C901
         )
     print("Executing codemod...", file=sys.stderr)
 
-    # We place results in this queue inside _parallel_exec_process_stub
-    # so that we can control when things get printed to the console.
-    queue = Queue()
-
-    if total == 1:
-        # Simple case, we should not pay for process overhead. Lets still
-        # use the exec stub however, so we can share code.
-        progress.print(0)
-        _parallel_exec_process_stub(
-            queue,
-            transform,
-            files[0],
-            repo_root,
-            unified_diff=unified_diff,
-            include_generated=include_generated,
-            generated_code_marker=generated_code_marker,
-            format_code=format_code,
-            formatter_args=formatter_args,
-            blacklist_patterns=blacklist_patterns,
-            python_version=python_version,
-        )
-        result = queue.get()
-        _print_parallel_result(
-            result,
-            progress,
-            unified_diff=bool(unified_diff),
-            show_successes=show_successes,
-            hide_generated=hide_generated,
-            hide_blacklisted=hide_blacklisted,
-        )
-        if isinstance(result.transform_result, TransformFailure):
-            return ParallelTransformResult(
-                successes=0,
-                failures=1,
-                skips=0,
-                warnings=len(result.transform_result.warning_messages),
-            )
-        elif isinstance(result.transform_result, (TransformSkip, TransformExit)):
-            return ParallelTransformResult(
-                successes=0,
-                failures=0,
-                skips=1,
-                warnings=len(result.transform_result.warning_messages),
-            )
-        elif isinstance(result.transform_result, TransformSuccess):
-            return ParallelTransformResult(
-                successes=1,
-                failures=0,
-                skips=0,
-                warnings=len(result.transform_result.warning_messages),
-            )
-        else:
-            raise Exception("Logic error, unaccounted for result!")
-
-    # Warm the parser, pre-fork.
-    parse_module(
-        "",
-        config=(
-            PartialParserConfig(python_version=python_version)
-            if python_version is not None
-            else PartialParserConfig()
-        ),
+    config = ExecutionConfig(
+        repo_root=repo_root,
+        unified_diff=unified_diff,
+        include_generated=include_generated,
+        generated_code_marker=generated_code_marker,
+        format_code=format_code,
+        formatter_args=formatter_args,
+        blacklist_patterns=blacklist_patterns,
+        python_version=python_version,
     )
 
-    # Complex case, more than one file
+    if total == 1:
+        # Simple case, we should not pay for process overhead.
+        # Let's just use a dummy synchronous pool.
+        jobs = 1
+        pool_impl = DummyPool
+    else:
+        pool_impl = Pool
+        # Warm the parser, pre-fork.
+        parse_module(
+            "",
+            config=(
+                PartialParserConfig(python_version=python_version)
+                if python_version is not None
+                else PartialParserConfig()
+            ),
+        )
+
     successes: int = 0
     failures: int = 0
     warnings: int = 0
     skips: int = 0
-    pending_processes: List[Process] = []
-
-    # Start processes
-    filename_to_process: Dict[str, Process] = {}
-    for f in files:
-        process = Process(
-            target=_parallel_exec_process_stub,
-            args=(
-                queue,
-                transform,
-                f,
-                repo_root,
-                unified_diff,
-                include_generated,
-                generated_code_marker,
-                format_code,
-                formatter_args,
-                blacklist_patterns,
-                python_version,
-            ),
-        )
-        pending_processes.append(process)
-        filename_to_process[f] = process
-
-    # Start the processes, allowing no more than num_processes to be running
-    # at once.
-    results_left = len(pending_processes)
-    joinable_processes: Set[Process] = set()
-    processes_started = 0
-
-    interrupted = False
-    while results_left > 0 and not interrupted:
-        while processes_started < jobs and pending_processes:
-            try:
-                # Move this process to the joinables
-                process = pending_processes.pop(0)
-                joinable_processes.add(process)
-
-                # Start it, bookkeep that we did
-                process.start()
-                processes_started += 1
-            except KeyboardInterrupt:
-                interrupted = True
-                continue
 
+    with pool_impl(processes=jobs) as p:  # type: ignore
+        args = [
+            {
+                "transformer": transform,
+                "filename": filename,
+                "config": config,
+            }
+            for filename in files
+        ]
         try:
-            result = queue.get(block=True, timeout=0.005)
-        except KeyboardInterrupt:
-            interrupted = True
-            continue
-        except Empty:
-            progress.print(successes + failures + skips)
-            continue
+            for result in p.imap_unordered(_execute_transform_wrap, args, chunksize=4):
+                # Print an execution result, keep track of failures
+                _print_parallel_result(
+                    result,
+                    progress,
+                    unified_diff=bool(unified_diff),
+                    show_successes=show_successes,
+                    hide_generated=hide_generated,
+                    hide_blacklisted=hide_blacklisted,
+                )
+                progress.print(successes + failures + skips)
 
-        # Bookkeep the result, since we know the process that returned this is done.
-        results_left -= 1
-        processes_started -= 1
+                if isinstance(result.transform_result, TransformFailure):
+                    failures += 1
+                elif isinstance(result.transform_result, TransformSuccess):
+                    successes += 1
+                elif isinstance(
+                    result.transform_result, (TransformExit, TransformSkip)
+                ):
+                    skips += 1
 
-        # Print an execution result, keep track of failures
-        _print_parallel_result(
-            result,
-            progress,
-            unified_diff=bool(unified_diff),
-            show_successes=show_successes,
-            hide_generated=hide_generated,
-            hide_blacklisted=hide_blacklisted,
-        )
-        progress.print(successes + failures + skips)
-
-        if isinstance(result.transform_result, TransformFailure):
-            failures += 1
-        elif isinstance(result.transform_result, TransformSuccess):
-            successes += 1
-        elif isinstance(result.transform_result, (TransformExit, TransformSkip)):
-            skips += 1
-
-        warnings += len(result.transform_result.warning_messages)
-
-        # Join the process to free any related resources.
-        # Remove all references to the process to allow the GC to
-        # clean up any file handles.
-        process = filename_to_process.pop(result.filename, None)
-        if process:
-            process.join()
-            joinable_processes.discard(process)
-
-    # Now, join on all of them so we don't leave zombies or hang
-    for p in joinable_processes:
-        p.join()
+                warnings += len(result.transform_result.warning_messages)
+        finally:
+            progress.clear()
 
     # Return whether there was one or more failure.
-    progress.clear()
-
-    # If we caught an interrupt, raise that
-    if interrupted:
-        raise KeyboardInterrupt()
     return ParallelTransformResult(
         successes=successes, failures=failures, skips=skips, warnings=warnings
     )
