@@ -10,7 +10,9 @@ from libcst.codemod import VisitorBasedCodemodCommand
 
 class UnnecessaryFormatString(VisitorBasedCodemodCommand):
 
-    DESCRIPTION: str = "Converts f-strings which perform no formatting to regular strings."
+    DESCRIPTION: str = (
+        "Converts f-strings which perform no formatting to regular strings."
+    )
 
     @m.leave(m.FormattedString(parts=(m.FormattedStringText(),)))
     def _check_formatted_string(
