@@ -36,6 +36,8 @@ from libcst.metadata.expression_context_provider import (
 )
 
 
+# Comprehensions are handled separately in _visit_comp_alike due to
+# the complexity of the semantics
 _ASSIGNMENT_LIKE_NODES = (
     cst.AnnAssign,
     cst.AsName,
@@ -43,7 +45,6 @@ _ASSIGNMENT_LIKE_NODES = (
     cst.AugAssign,
     cst.ClassDef,
     cst.CompFor,
-    cst.For,
     cst.FunctionDef,
     cst.Global,
     cst.Import,
@@ -123,7 +124,7 @@ class Access:
             for assignment in assignments
             if assignment.scope != self.scope or assignment._index < self.__index
         }
-        if not previous_assignments and assignments:
+        if not previous_assignments and assignments and self.scope.parent != self.scope:
             previous_assignments = self.scope.parent[name]
         self.__assignments |= previous_assignments
 
@@ -782,9 +783,12 @@ class ScopeVisitor(cst.CSTVisitor):
             return False
         if "typing.cast" in qnames:
             node.func.visit(self)
-            self.__in_type_hint.add(node)
             if len(node.args) > 0:
+                self.__in_type_hint.add(node)
                 node.args[0].visit(self)
+                self.__in_type_hint.discard(node)
+                for arg in node.args[1:]:
+                    arg.visit(self)
             return False
         return True
 
@@ -820,11 +824,12 @@ class ScopeVisitor(cst.CSTVisitor):
         return False
 
     def visit_Subscript(self, node: cst.Subscript) -> Optional[bool]:
-        qnames = {qn.name for qn in self.scope.get_qualified_names_for(node.value)}
-        if any(qn.startswith(("typing.", "typing_extensions.")) for qn in qnames):
-            self.__in_type_hint.add(node)
-        if "typing.Literal" in qnames or "typing_extensions.Literal" in qnames:
-            self.__in_ignored_subscript.add(node)
+        if isinstance(node.value, cst.Name):
+            qnames = {qn.name for qn in self.scope.get_qualified_names_for(node.value)}
+            if any(qn.startswith(("typing.", "typing_extensions.")) for qn in qnames):
+                self.__in_type_hint.add(node)
+            if "typing.Literal" in qnames or "typing_extensions.Literal" in qnames:
+                self.__in_ignored_subscript.add(node)
         return True
 
     def leave_Subscript(self, original_node: cst.Subscript) -> None:
@@ -973,6 +978,8 @@ class ScopeVisitor(cst.CSTVisitor):
         self.provider.set_metadata(for_in, self.scope)
         with self._new_scope(ComprehensionScope, node):
             for_in.target.visit(self)
+            # Things from here on can refer to the target.
+            self.scope._assignment_count += 1
             for condition in for_in.ifs:
                 condition.visit(self)
             inner_for_in = for_in.inner_for_in
@@ -985,6 +992,14 @@ class ScopeVisitor(cst.CSTVisitor):
                 node.elt.visit(self)
         return False
 
+    def visit_For(self, node: cst.For) -> Optional[bool]:
+        node.target.visit(self)
+        self.scope._assignment_count += 1
+        for child in [node.iter, node.body, node.orelse, node.asynchronous]:
+            if child is not None:
+                child.visit(self)
+        return False
+
     def infer_accesses(self) -> None:
         # Aggregate access with the same name and batch add with set union as an optimization.
         # In worst case, all accesses (m) and assignments (n) refer to the same name,
