@@ -4,6 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 
 
+import sys
 from textwrap import dedent
 from typing import Mapping, Tuple, cast
 
@@ -1567,3 +1568,122 @@ class ScopeProviderTest(UnitTest):
         self.assertEqual(list(a_param_refs)[0].node, comp.for_in.iter)
         self.assertEqual(len(a_comp_assignment.references), 1)
         self.assertEqual(list(a_comp_assignment.references)[0].node, comp.elt)
+
+    def test_for_scope_ordering(self) -> None:
+        m, scopes = get_scope_metadata_provider(
+            """
+            def f():
+                for x in []:
+                    x
+            class X:
+                def f():
+                    for x in []:
+                        x
+            """
+        )
+        for scope in scopes.values():
+            for acc in scope.accesses:
+                self.assertEqual(
+                    len(acc.referents),
+                    1,
+                    msg=(
+                        "Access for node has incorrect number of referents: "
+                        + f"{acc.node}"
+                    ),
+                )
+
+    def test_no_out_of_order_references_in_global_scope(self) -> None:
+        m, scopes = get_scope_metadata_provider(
+            """
+            x = y
+            y = 1
+            """
+        )
+        for scope in scopes.values():
+            for acc in scope.accesses:
+                self.assertEqual(
+                    len(acc.referents),
+                    0,
+                    msg=(
+                        "Access for node has incorrect number of referents: "
+                        + f"{acc.node}"
+                    ),
+                )
+
+    def test_walrus_accesses(self) -> None:
+        if sys.version_info < (3, 8):
+            self.skipTest("This python version doesn't support :=")
+        m, scopes = get_scope_metadata_provider(
+            """
+            if x := y:
+                y = 1
+                x
+            """
+        )
+        for scope in scopes.values():
+            for acc in scope.accesses:
+                self.assertEqual(
+                    len(acc.referents),
+                    1 if getattr(acc.node, "value") == "x" else 0,
+                    msg=(
+                        "Access for node has incorrect number of referents: "
+                        + f"{acc.node}"
+                    ),
+                )
+
+    def test_cast(self) -> None:
+        with self.assertRaises(cst.ParserSyntaxError):
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import TypeVar
+                TypeVar("Name", "3rr0r")
+                """
+            )
+
+        try:
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import TypeVar
+                TypeVar("3rr0r", "int")
+                """
+            )
+        except cst.ParserSyntaxError:
+            self.fail(
+                "First string argument of NewType and TypeVar should not be parsed"
+            )
+
+        with self.assertRaises(cst.ParserSyntaxError):
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import Dict
+                Dict["str", "3rr0r"]
+                """
+            )
+
+        try:
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import Dict, cast
+                cast(Dict[str, str], {})["3rr0r"]
+                """
+            )
+        except cst.ParserSyntaxError:
+            self.fail("Subscript of function calls should not be parsed")
+
+        try:
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import cast
+                cast(str, "3rr0r")
+                """
+            )
+        except cst.ParserSyntaxError:
+            self.fail("String arguments of cast should not be parsed")
+
+        with self.assertRaises(cst.ParserSyntaxError):
+            m, scopes = get_scope_metadata_provider(
+                """
+                from typing import cast
+                cast("3rr0r", "")
+                """
+            )
